import * as i0 from '@angular/core';
import { EventEmitter, forwardRef, Component, ViewEncapsulation, ChangeDetectionStrategy, Input, Output, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Annotation, EditorState, StateEffect } from '@codemirror/state';
import { oneDark } from '@codemirror/theme-one-dark';
import { placeholder } from '@codemirror/view';
import { basicSetup, EditorView } from 'codemirror';

const External = Annotation.define();
class CodeEditor {
    /** Editor's value. */
    get value() {
        return this._value;
    }
    set value(newValue) {
        this._value = newValue;
        this.setValue(newValue);
    }
    constructor(_elementRef) {
        this._elementRef = _elementRef;
        this._value = '';
        /** Editor's theme. */
        this.theme = 'light';
        /** Editor's placecholder. */
        this.placeholder = '';
        /** Whether the editor is disabled.  */
        this.disabled = false;
        /** Whether the editor is readonly. */
        this.readonly = false;
        /** Whether focus on the editor when init. */
        this.autoFocus = false;
        /**
         * EditorState's [extensions](https://codemirror.net/docs/ref/#state.EditorStateConfig.extensions).
         * It includes the [basicSetup](https://codemirror.net/docs/ref/#codemirror.basicSetup) by default.
         */
        this.extensions = [basicSetup];
        /** Event emitted when the editor's value changes. */
        this.change = new EventEmitter();
        /** Event emitted when focus on the editor. */
        this.focus = new EventEmitter();
        /** Event emitted when the editor has lost focus. */
        this.blur = new EventEmitter();
        this._onChange = () => { };
        this._onTouched = () => { };
        /** Register a function to be called every time the view updates. */
        this._updateListener = EditorView.updateListener.of(vu => {
            if (vu.docChanged && !vu.transactions.some(tr => tr.annotation(External))) {
                const value = vu.state.doc.toString();
                this._onChange(value);
                this.change.emit(value);
            }
        });
    }
    /** Get the extensions of the editor. */
    getExtensions() {
        const basicExtensions = [
            this._updateListener,
            EditorView.editable.of(!this.disabled),
            EditorState.readOnly.of(this.readonly),
            placeholder(this.placeholder),
        ];
        if (this.theme == 'light') {
            // nothing to do
        }
        else if (this.theme == 'dark') {
            basicExtensions.push(oneDark);
        }
        else {
            basicExtensions.push(this.theme);
        }
        return basicExtensions.concat(this.extensions);
    }
    /** Reconfigure the root extensions of the editor. */
    reconfigure() {
        this.view?.dispatch({
            effects: StateEffect.reconfigure.of(this.getExtensions()),
        });
    }
    /** Sets the editor's value. */
    setValue(value) {
        this.view?.dispatch({
            changes: { from: 0, to: this.view.state.doc.length, insert: value },
        });
    }
    ngOnChanges(changes) {
        if (this.view) {
            this.reconfigure();
        }
    }
    ngOnInit() {
        this.view = new EditorView({
            root: this.root,
            parent: this._elementRef.nativeElement,
            state: EditorState.create({
                doc: this.value,
                extensions: this.getExtensions(),
            }),
        });
        if (this.autoFocus) {
            this.view?.focus();
        }
        this.view?.contentDOM.addEventListener('focus', () => {
            this._onTouched();
            this.focus.emit();
        });
        this.view?.contentDOM.addEventListener('blur', () => {
            this._onTouched();
            this.blur.emit();
        });
    }
    ngOnDestroy() {
        this.view?.destroy();
        this.view = null;
    }
    writeValue(value) {
        if (this.view) {
            this.setValue(value);
        }
    }
    registerOnChange(fn) {
        this._onChange = fn;
    }
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
        this.reconfigure();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.9", ngImport: i0, type: CodeEditor, deps: [{ token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "17.3.9", type: CodeEditor, isStandalone: true, selector: "code-editor", inputs: { root: "root", value: "value", theme: "theme", placeholder: "placeholder", disabled: "disabled", readonly: "readonly", autoFocus: "autoFocus", extensions: "extensions" }, outputs: { change: "change", focus: "focus", blur: "blur" }, providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => CodeEditor),
                multi: true,
            },
        ], usesOnChanges: true, ngImport: i0, template: ``, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.9", ngImport: i0, type: CodeEditor, decorators: [{
            type: Component,
            args: [{
                    selector: 'code-editor',
                    standalone: true,
                    template: ``,
                    encapsulation: ViewEncapsulation.None,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(() => CodeEditor),
                            multi: true,
                        },
                    ],
                }]
        }], ctorParameters: () => [{ type: i0.ElementRef }], propDecorators: { root: [{
                type: Input
            }], value: [{
                type: Input
            }], theme: [{
                type: Input
            }], placeholder: [{
                type: Input
            }], disabled: [{
                type: Input
            }], readonly: [{
                type: Input
            }], autoFocus: [{
                type: Input
            }], extensions: [{
                type: Input
            }], change: [{
                type: Output
            }], focus: [{
                type: Output
            }], blur: [{
                type: Output
            }] } });

class CodeEditorModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.9", ngImport: i0, type: CodeEditorModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "17.3.9", ngImport: i0, type: CodeEditorModule, imports: [CodeEditor], exports: [CodeEditor] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "17.3.9", ngImport: i0, type: CodeEditorModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.9", ngImport: i0, type: CodeEditorModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CodeEditor],
                    exports: [CodeEditor],
                }]
        }] });

/*
 * Public API Surface of code-editor
 */

/**
 * Generated bundle index. Do not edit.
 */

export { CodeEditor, CodeEditorModule };
//# sourceMappingURL=acrodata-code-editor.mjs.map
